package org.once.serviceChoicer.lingo;
import com.lindo.Lingd90;

public class LingoSolver
{
   // Load the Lingo JNI interface
   static {
      System.loadLibrary( "Lingj90");
   }

   // Stores the Lingo JNI environment pointer
   private int pnLngEnv;
   private int nLastIterationCount;
   private String filename;
   private double[] obj ;
   
   public LingoSolver(String filename) throws Exception
   {
	   this.filename = filename;
	   obj = new double[1];
	   solve();
   }

   private static int MySolverCallback( int pnLng, int iLoc, Object jobj)
   {
      LingoSolver s = (LingoSolver) jobj;
      int nIterations[] = new int [1];
	  Lingd90.LSgetCallbackInfoIntLng( s.pnLngEnv, Lingd90.LS_IINFO_ITERATIONS_LNG, nIterations);
      if ( nIterations[0] != s.nLastIterationCount)
      {
         s.nLastIterationCount = nIterations[0];
         System.out.println("In Java callback function...iterations = " + s.nLastIterationCount);
      }
      return(0);
   }

   private static int MyErrorCallback( int pnLng, Object jobj, int nErrorCode, String jsErrMessage)
   {
      System.out.println("In Java callback function...Lingo error code = " + nErrorCode);
      System.out.println("In Java callback function...Lingo error message:\n\n " + jsErrMessage);
      return( 0);
   }

   private void solve() throws Exception
   {

      int nErr;
      double status[] = new double [1];
      
      System.out.println("\nSolving...");            
      // create the Lingo environment
      pnLngEnv = Lingd90.LScreateEnvLng();
      if ( pnLngEnv == 0)
      {
         System.out.println( "***Unable to create Lingo environment***");
         throw new Exception("Unable to create Lingo environment");
//         return;
      }
      
      // open a log file
      nErr = Lingd90.LSopenLogFileLng( pnLngEnv, "lingo.log");
      if ( nErr != Lingd90.LSERR_NO_ERROR_LNG )
      {
         System.out.println( "***LSopenLogFileLng() error***: " + nErr);
//         return;
         throw new Exception("LSopenLogFileLng() error");
      }
      
      // pass lingo a pointer to the staffing needs
      int[] nPointersNow = new int[1];
	  nErr = Lingd90.LSsetPointerLng( pnLngEnv, status, nPointersNow);
      if ( nErr != Lingd90.LSERR_NO_ERROR_LNG )
      {
         System.out.println( "***LSsetPointerLng() error***: " + nErr);
//         return;
         throw new Exception("LSsetPointerLng() error");
      }
      
      nErr = Lingd90.LSsetPointerLng( pnLngEnv, obj, nPointersNow);
      if ( nErr != Lingd90.LSERR_NO_ERROR_LNG )
      {
         System.out.println( "***LSsetPointerLng() error***: " + nErr);
         throw new Exception("LSsetPointerLng() error");
//         return;
      }
      
      // pass Lingo the name of the solver callback function...
      nErr = Lingd90.LSsetCallbackSolverLng( pnLngEnv, "MySolverCallback", this);
      
      // ...and the error callback function
      nErr = Lingd90.LSsetCallbackErrorLng( pnLngEnv, "MyErrorCallback", this);
      
      // construct the script
      
      // echo input to log file
      String sScript = "";
      sScript += "SET ECHOIN 1" + "\n";
      sScript += "SET LINLEN 200\n";
      // load the model from disk
      sScript = sScript + "TAKE " + filename + "\n";
      
      // view the formulation
      sScript = sScript + "LOOK ALL" + "\n";
      
      // solve
      sScript = sScript + "GO" + "\n";
      
      // exit script processor
      sScript = sScript + "QUIT" + "\n";
      
      // run the script
      status[0] = -1;
      nLastIterationCount = -1;
	  nErr = Lingd90.LSexecuteScriptLng( pnLngEnv, sScript);
      if ( nErr != Lingd90.LSERR_NO_ERROR_LNG )
      {
         System.out.println( "***LSexecuteScriptLng error***: " + nErr);
         throw new Exception("LSexecuteScriptLng error");
//         return;
      }
      
      // clear the pointers to force update of local arrays
      // ***NOTE*** solution won't get passed to local arrays until
      // LSclearPointersLng is called!!!
	  nErr = Lingd90.LSclearPointersLng( pnLngEnv);
      if ( nErr != Lingd90.LSERR_NO_ERROR_LNG )
      {
         System.out.println( "***LSclearPointerLng() error***: " + nErr);
//         return;
         throw new Exception("LSclearPointerLng() error");
      }
      
      
      // check the solution status
      if ( status[0] != Lingd90.LS_STATUS_GLOBAL_LNG) {
    	System.out.println( "***Unable to Solve*** status:" + status[0]);
    	throw new Exception("Unable to Solve");
      }
      // close Lingo's log file
	  nErr = Lingd90.LScloseLogFileLng( pnLngEnv);
      if ( nErr != Lingd90.LSERR_NO_ERROR_LNG )
      {
         System.out.println( "***LScloseLogFileLng() error***: " + nErr);
         throw new Exception("LScloseLogFileLng() error");
//         return;
      }
	  
      // delete the Lingo environment
		Lingd90.LSdeleteEnvLng(pnLngEnv);
	}

	public double getObj() {
		return obj[0];
	}
}

