package org.once.serviceChoicer.output.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.once.serviceChoicer.candidate.ICandidate;
import org.once.serviceChoicer.input.process.IEPath;
import org.once.serviceChoicer.input.qos.IQoSAttr.Type;
import org.once.serviceChoicer.math.IVariable;
import org.once.serviceChoicer.math.impl.Monomial;
import org.once.serviceChoicer.math.impl.Variable;
import org.once.serviceChoicer.output.IVarManager;

public class VarManager implements IVarManager{
	
	private Map<Integer,Map<Type, IVariable>> epathTypeMap;
	private Map<String, IVariable> candiVarMap;
	private MonVarMap monVarMap;
	
	private List<IVariable> varList;
	
	private Map<Type, IVariable> qosScoreVarMap;
	private Map<Type, IVariable> qosVarMap;

	public VarManager(){
		init();
	}
	
	private void init(){
		this.epathTypeMap = new HashMap<Integer,Map<Type, IVariable>>();
		this.candiVarMap = new HashMap<String,IVariable>();
		this.monVarMap = new MonVarMap();
		this.varList = new ArrayList<IVariable>();
		
		qosScoreVarMap = new HashMap<Type, IVariable>();
		qosVarMap = new HashMap<Type, IVariable>();
	}

	public IVariable getVar(IEPath epath, Type type) {
		if(this.epathTypeMap.containsKey(epath.getPathNo())){
			Map<Type,IVariable> map = epathTypeMap.get(epath.getPathNo());
			if(!map.containsKey(type)){ 
				addVar(map, epath, type);
			}
			return map.get(type);
			
		}else{
			Map<Type, IVariable> map = new HashMap<Type, IVariable>();
			addVar(map, epath, type);
			epathTypeMap.put(epath.getPathNo(), map);
			return map.get(type);
		}
	}
	
	private void addVar(Map<Type, IVariable> map,IEPath epath, Type type){
		IVariable var = new Variable(epath,type);
		map.put(type, var);
		varList.add(var);
	}

/*	public IVariable getVar(ICandidate candidate) {
		String key = candidate.getName();
		if(!candiVarMap.containsKey(key)){
			IVariable var = new Variable(candidate);
			candiVarMap.put(key, var);
			varList.add(var);
		}
		return (IVariable) candiVarMap.get(key);
	}
	*/
	public IVariable getVar(ICandidate candidate) {
		String key = candidate.getName()+ candidate.getServiceID();
		if(!candiVarMap.containsKey(key)){
			IVariable var = new Variable(candidate);
			candiVarMap.put(key, var);
			varList.add(var);
		}
		return (IVariable) candiVarMap.get(key);
	}

	public IVariable getVar(Monomial monomial) {
		if(!this.monVarMap.containsKey(monomial)){
			IVariable var = new Variable(monomial);
			monVarMap.put(monomial.clone(), var);
			this.varList.add(var);
		} 
		return monVarMap.getKey(monomial);
	}

	class MonVarMap {
		private ArrayList<Monomial> mList;
		private ArrayList<IVariable> varList;
		private int keyNum;
		
		public MonVarMap(){
			this.mList = new ArrayList<Monomial>();
			this.varList = new ArrayList<IVariable>();
			keyNum = -1;
		}
		
		public IVariable get(int i) {
			return this.varList.get(i);
		}

		public int size() {
			return this.mList.size();
		}

		public boolean containsKey(Monomial monomial) {
			for(int i = 0; i < mList.size(); i++){
				if(mList.get(i).canUnit(monomial)){
					keyNum = i;
					return true;
				}
			}
			return false;
		}
		
		public void put(Monomial monomial, IVariable variable) {
			mList.add(monomial);
			varList.add(variable);
		}
		
		public IVariable getKey(Monomial monomial){
			if(containsKey(monomial))
				return varList.get(keyNum);
			else
				return null;
		}
		
	}

	public List<IVariable> getVarList() {
		return this.varList;
	}

	@Override
	public boolean containsKey(Monomial monomial) {
		return this.monVarMap.containsKey(monomial);
	}
	
	/**
	 * ݷQOSԵͣɱisScoreΪtureʾΪQOSɱΪQOSɱ
	 */
	public IVariable getVar(Type type, boolean isScore) {
		IVariable var = null;
		if(isScore){
			var = getVar(qosScoreVarMap,type,true);			
		}else{
			var = getVar(qosVarMap,type, false);
		}
		return var;
	}
	
	private IVariable getVar(Map<Type, IVariable> map,Type type, boolean isScore){
		if(!map.containsKey(type)){
			IVariable var = new Variable(type, isScore);
			map.put(type, var);
			varList.add(var);
		}
		return map.get(type);
	}
	
	public IVariable getVar(){
		IVariable var = new Variable();
		this.varList.add(var);
		return var;
	}

/*	@Override
	public String getCandidate(IVariable cand) {
		String result = null;
		for(String key : candiVarMap.keySet()){
			if(candiVarMap.get(key).equals(cand)){
				result = key;
			}
		}
		return result;
	}
	*/
	
	

}
