package org.once.serviceChoicer.system.impl;

import java.util.ArrayList;
import java.util.List;

import org.once.serviceChoicer.candidate.ICandidate;
import org.once.serviceChoicer.candidate.IRule;
import org.once.serviceChoicer.candidate.IUDDI;
import org.once.serviceChoicer.exception.ServiceChoicerException;
import org.once.serviceChoicer.input.process.IEPath;
import org.once.serviceChoicer.input.process.IService;
import org.once.serviceChoicer.input.process.ISubPath;
import org.once.serviceChoicer.input.qos.IQoSAttr.Type;
import org.once.serviceChoicer.math.IPolynomial;
import org.once.serviceChoicer.math.IVariable;
import org.once.serviceChoicer.math.impl.Polynomial;
import org.once.serviceChoicer.output.IConstrain;
import org.once.serviceChoicer.output.IVarManager;
import org.once.serviceChoicer.output.impl.QoSConstrain;
import org.once.serviceChoicer.qosDescrip2.parser.IParser;
import org.once.serviceChoicer.qosDescrip2.parser.abstr.ATokenParser4log4qos;
import org.once.serviceChoicer.qosDescrip2.parser.impl.OperatorFirstAnalyzer;
import org.once.serviceChoicer.qosDescrip2.parser.impl.RelationTableTest4log;
import org.once.serviceChoicer.qosDescrip2.parser.impl.StipulatTable4log;
import org.once.serviceChoicer.qosDescrip2.parser.impl.TokenParser4QoS4PN;
import org.once.serviceChoicer.qosDescrip2.token.impl.LAttr4log4Pn;
import org.once.serviceChoicer.system.IAggrCaculator;
import org.once.serviceChoicer.system.ICaculModle;
import org.once.serviceChoicer.system.util.IAggrTable;
import org.once.serviceChoicer.system.util.IAggregation.OPType;

public class AggrCaculator implements IAggrCaculator{
	
	private IUDDI uddi;
	private IVarManager varManager;
	private IAggrTable aggrTable;

	public AggrCaculator(IUDDI uddi, IVarManager varManager,
			IAggrTable aggrTable) {
		this.uddi = uddi;
		this.varManager = varManager;
		this.aggrTable = aggrTable;
	}

	/**
	 * (non-Javadoc)
	 * @see org.once.serviceChoicer.system.IAggrCaculator#getConstrains(org.once.serviceChoicer.process.IEPath, org.once.serviceChoicer.process.IQoSAttr.Type)
	 * ePath,typeɶӦԼʽ
	 */

	public List<IConstrain> getConstrains(IEPath ePath, Type type) throws Exception {
		List<IConstrain> qosCList = new ArrayList<IConstrain>();
		
		// ִ·еĿѡ
		List<ICandidate> candidates = new ArrayList<ICandidate>();
		for(IService service:ePath.getServices()){
			candidates.addAll(uddi.getCandidates(service));
		}
		
		
		ATokenParser4log4qos tokenParser = new TokenParser4QoS4PN(candidates,varManager);

		RelationTableTest4log relationTable = new RelationTableTest4log();
		StipulatTable4log sTable = new StipulatTable4log(tokenParser);

		OperatorFirstAnalyzer rRuleParser = new OperatorFirstAnalyzer(
				relationTable, sTable, tokenParser);
		
		
		// 󲿱
		IVariable var = varManager.getVar(ePath, type);
		IConstrain.OPType opType;
		IPolynomial pn = null;
		
		// ɷźҲʽ
		OPType ePathOp = aggrTable.getAggrOp(type).getEPathOp();
		
		if(ePathOp == OPType.PRODUCT 
				|| ePathOp == OPType.SUM){
			opType = IConstrain.OPType.EQUAL;	
			pn = new RelationModel(ePath.getServices(), type, rRuleParser).getPNomial();
            qosCList.add(new QoSConstrain(var,opType,pn));
            return qosCList;
		}
		if(ePathOp == OPType.MAX 
				|| ePathOp == OPType.MIN)
		{
			if(ePathOp == OPType.MAX){
				opType = IConstrain.OPType.GT;
			}
			else {
				opType = IConstrain.OPType.LT;
			}
			for(int i = 0; i < ePath.getSubPathes().size(); i++){
				ISubPath subPath = (ISubPath) ePath.getSubPathes().get(i);
				pn = new RelationModel(subPath.getServices(), type, rRuleParser).getPNomial();
				qosCList.add(new QoSConstrain(var,opType,pn));
			}
			return qosCList;
		}
		return null;
	}
	
	
	/*
	 * 
	 */
	class RelationModel implements ICaculModle{
		
		private List<IService> services;
		private Type type;
		private OperatorFirstAnalyzer rRuleParser;

		public RelationModel(List<IService> services, Type type, OperatorFirstAnalyzer ruleParser) {
			this.services = services;
			this.type = type;
			this.rRuleParser = (OperatorFirstAnalyzer) ruleParser;
		}

		public IPolynomial getPNomial() throws ServiceChoicerException {
			IPolynomial result = new Polynomial();
			for(int i = 0; i < services.size(); i++){
				List<ICandidate> cList = uddi.getCandidates(services.get(i));
				for(int j = 0; j < cList.size(); j++){
					IVariable cVar = varManager.getVar(cList.get(j));
					IPolynomial item = new Polynomial(cVar);
					item.multi(getCQoSRule(cList.get(j)));
					result.add(item);
				}
			}
			return result;
			
		}
		
		private IPolynomial getCQoSRule(ICandidate candidate) {
			IPolynomial result = new Polynomial();
			IPolynomial sub = null;
			
			List<IRule> rules = candidate.getRules(type);
			
			int defaultNo = -1;
			double defaultVaule = 0;
			for(int i = 0; i < rules.size(); i++){
				IRule rule = rules.get(i);
				double value = rule.getVar();
				if(aggrTable.isProduct(type)){
					value = Math.log(value);
				}
				String rRule = rule.getRRule();
				
				if(rRule == null){
					defaultNo = i;
					defaultVaule = value;
				}else{
					LAttr4log4Pn attr = (LAttr4log4Pn)this.rRuleParser.parse(rRule);
//					
					if(attr == null)
						continue;
					
					IPolynomial item = attr.getLAttrValue();
					if(item == null || item.isEmpty())
						continue;
					if(aggrTable.isProduct(type)){
						if(sub == null)
							sub = new Polynomial(1);
						sub = sub.multi(item.clone());
					}else{
						if(sub == null)
							sub = new Polynomial();
						sub = sub.add(item.clone());
					}
					result.add(item.multiCoef(value));
				}
				
			}

			if(defaultNo == -1){
				System.out.println("error");
				result = null;
			}
			if(sub== null ||sub.isEmpty())
				result = new Polynomial(defaultVaule);
			else{
				sub.multiCoef(-1);
				sub.add(new Polynomial(1));
				sub.multiCoef(defaultVaule);			
				result.add(sub);
//				System.out.println("result: " + result.toString());
			}
			return result;
		}
	}

}
